<?php
/**
 *    This file is part of Mobile Assistant Connector.
 *
 *   Mobile Assistant Connector is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   Mobile Assistant Connector is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Mobile Assistant Connector.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  @author    eMagicOne <contact@emagicone.com>
 *  @copyright 2014-2018 eMagicOne
 *  @license   http://www.gnu.org/licenses   GNU General Public License
 */

class AttributeGenerator
{

    private $product;

    /**
     * AttributeGenerator constructor.
     */
    public function __construct($product)
    {
        $this->product = $product;
    }

    public function processGenerate($product, $options)
    {
        $prepared_options = array_values($options);
        if (count($prepared_options) && Validate::isLoadedObject($this->product)) {
            $this->setAttributesImpacts($this->product->id, $prepared_options);
            $combinations = array_values($this->createCombinations($prepared_options));
            $values = array_values(array_map(array($this, 'addAttribute'), $combinations));

            // @since 1.5.0
            if ($this->product->depends_on_stock == 0) {
                $attributes = Product::getProductAttributesIds($this->product->id, true);
                foreach ($attributes as $attribute) {
                    StockAvailable::removeProductFromStockAvailable(
                        $this->product->id,
                        $attribute['id_product_attribute'],
                        Context::getContext()->shop
                    );
                }
            }

            $product->deleteProductAttributes();
            $product->generateMultipleCombinations($values, $combinations);
        }
    }

    protected function addAttribute()
    {
        if ($this->product->id) {
            return array(
                'id_product' => (int)$this->product->id,
                'price' => (float) 0,
                'weight' => (float) 0,
                'ecotax' => 0,
                'quantity' => 0,
                'minimal_quantity' => 0,
                'reference' => pSQL(''),
                'default_on' => 0,
                'available_date' => '0000-00-00'
            );
        }
        return array();
    }

    protected static function createCombinations($list)
    {
        if (count($list) <= 1) {
            return count($list)
                ? array_map(create_function('$v', 'return (array($v));'), $list[0])
                : $list;
        }
        $res = array();
        $first = array_pop($list);
        foreach ($first as $attribute) {
            $tab = self::createCombinations($list);
            foreach ($tab as $to_add) {
                $res[] = is_array($to_add) ? array_merge($to_add, array($attribute)) : array($to_add, $attribute);
            }
        }
        return $res;
    }

    protected static function setAttributesImpacts($id_product, $prepared_options)
    {
        $attributes = array();
        foreach ($prepared_options as $group) {
            foreach ($group as $attribute) {
                $attributes[] =
                    '(' . (int)$id_product . ', ' . (int)$attribute . ', ' . (float)0 . ', ' . (float)0 . ')';
            }
        }

        return Db::getInstance()->execute('
		INSERT INTO `'._DB_PREFIX_.'attribute_impact` (`id_product`, `id_attribute`, `price`, `weight`)
		VALUES '.implode(',', $attributes).'
		ON DUPLICATE KEY UPDATE `price` = VALUES(price), `weight` = VALUES(weight)');
    }
}
