<?php
/**
 *    This file is part of Mobile Assistant Connector.
 *
 *   Mobile Assistant Connector is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   Mobile Assistant Connector is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Mobile Assistant Connector.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  @author    eMagicOne <contact@emagicone.com>
 *  @copyright 2014-2018 eMagicOne
 *  @license   http://www.gnu.org/licenses   GNU General Public License
 */

function isAuthenticated($key)
{
    if (hash('sha256', _COOKIE_KEY_) == $key) {
        return true;
    }

    return false;
}

function changeStatus($ids, $value)
{
    $ids = prepareIds($ids);

    if (!$ids) {
        return 'Parameters are incorrect';
    }

    $result = Db::getInstance()->update(
        'mobassistantconnector_accounts',
        array('status' => (int)$value),
        "`id` IN ($ids)"
    );

    if ($result) {
        return 'success';
    }

    return 'Some error occurred';
}

function changeStatusForUser($user_ids, $value)
{
    $ids = prepareIds($user_ids);

    if (!$ids) {
        return 'Parameters are incorrect';
    }

    $result = Db::getInstance()->update(
        'mobassistantconnector_users',
        array('status' => (int)$value),
        '`user_id` IN ('.$ids.')'
    );

    if ($result) {
        return 'success';
    }

    return 'Some error occurred while changing users status in bulk';
}

function deletePush($ids)
{
    $ids = prepareIds($ids);

    if (!$ids) {
        return 'Parameters are incorrect';
    }

    $result = Db::getInstance()->delete('mobassistantconnector_push_notifications', '`id` IN ('.$ids.')');
    deleteEmptyDevices();
    deleteEmptyAccounts();

    if ($result) {
        return 'success';
    }

    return 'Some error occurred while deleting push notification data';
}

function deleteUser($user_ids)
{
    $user_ids = prepareIds($user_ids);

    if (!$user_ids) {
        return 'Parameters are incorrect';
    }

    $result1 = Db::getInstance()->delete('mobassistantconnector_users', '`user_id` IN ('.$user_ids.')');
    $result2 = Db::getInstance()->delete('mobassistantconnector_session_keys', '`user_id` IN ('.$user_ids.')');
    $result3 = Db::getInstance()->delete('mobassistantconnector_push_notifications', '`user_id` IN ('.$user_ids.')');
    deleteEmptyDevices();
    deleteEmptyAccounts();

    if ($result1 && $result2 && $result3) {
        return 'success';
    }

    return 'Some error occurred while deleting user';
}

function getUsers()
{
    $users_obj = new DbQuery();
    $users_obj->select(
        'mu.`user_id`,
        mu.`username`,
        mu.`password`,
        mu.`allowed_actions`,
        mu.`qr_code_hash`,
        mu.`status`'
    );
    $users_obj->from('mobassistantconnector_users', 'mu');
    $users_sql = $users_obj->build();
    $users = Db::getInstance()->executeS($users_sql);

    if (!$users) {
        $users = array();
    }

    $users_count = count($users);
    $shop_url    = getShopUrl();

    for ($i = 0; $i < $users_count; $i++) {
        $users[$i]['allowed_actions'] = getUserActionsFromCodes($users[$i]['allowed_actions']);
        $users[$i]['qr_code_data'] = call_user_func(
            'base64_encode',
            Tools::jsonEncode(
                array('url' => $shop_url, 'login' => $users[$i]['username'], 'password' => $users[$i]['password'])
            )
        );
        $users[$i]['pushes'] = getPushesByUserId($users[$i]['user_id']);
    }

    return $users;
}

function getUserPushes($user_id)
{
    $user_ids = prepareIds($user_id);

    if (!$user_ids) {
        return 'Parameters are incorrect';
    }

    return getPushesByUserId($user_id);
}

function getUnlinkedPushes()
{
    return getPushesByUserId(false);
}

function getPushesByUserId($user_id)
{
    if ($user_id === false) {
        $where = 'mpn.`user_id` IS NULL OR mpn.`user_id` NOT IN (SELECT `user_id` FROM `'._DB_PREFIX_
            .'mobassistantconnector_users`)';
    } else {
        $where = 'mpn.`user_id` = '.(int)$user_id;
    }

    $devices_obj = new DbQuery();
    $devices_obj->select(
        'mpn.`id`,
        mpn.`id_shop`,
        mpn.`app_connection_id`,
        mpn.`device_unique_id`,
        mpn.`new_order`,
        mpn.`new_customer`,
        mpn.`order_statuses`,
        ma.`id` AS account_id,
        ma.`status`,
        ma.`account_email`,
        md.`device_name`,
        md.`last_activity`,
        c.`iso_code` AS currency_iso'
    );
    $devices_obj->from('mobassistantconnector_push_notifications', 'mpn');
    $devices_obj->leftJoin('mobassistantconnector_devices', 'md', 'md.`device_unique_id` = mpn.`device_unique_id`');
    $devices_obj->leftJoin('mobassistantconnector_accounts', 'ma', 'ma.`id` = md.`account_id`');
    $devices_obj->leftJoin('currency', 'c', 'c.`id_currency` = mpn.`currency_code`');
    $devices_obj->where($where);
    $devices_sql = $devices_obj->build();
    $devices = Db::getInstance()->executeS($devices_sql);

    if (!$devices) {
        $devices = array();
    }

    $devices = replaceNull($devices);
    $devices = formDevices($devices);

    return $devices;
}

function getActions($user_id)
{
    $user_actions = Db::getInstance()->getValue(
        'SELECT `allowed_actions` FROM `'._DB_PREFIX_.'mobassistantconnector_users` WHERE `user_id` = '.$user_id
    );

    if ($user_actions === false) {
        return 'error';
    }

    return getActionsByCodes($user_actions);
}

function saveUserActions($user_id, $selected_actions)
{
    $result = Db::getInstance()->update(
        'mobassistantconnector_users',
        array('allowed_actions' => $selected_actions),
        '`user_id` = '.$user_id
    );

    $allowed_actions = explode(';', $selected_actions);
    doOperationsAfterUserActionsSaved($user_id, $allowed_actions);

    if (!$result) {
        return 'error';
    }

    return 'success';
}

function doOperationsAfterUserActionsSaved($user_id, $allowed_actions)
{
    $new_order      = true;
    $new_customer   = true;
    $order_statuses = true;
    $data           = array();

    if (!in_array('push_notification_settings_new_order', $allowed_actions)) {
        $new_order = false;
        $data['new_order'] = 0;
    }

    if (!in_array('push_notification_settings_new_customer', $allowed_actions)) {
        $new_customer = false;
        $data['new_customer'] = 0;
    }

    if (!in_array('push_notification_settings_order_statuses', $allowed_actions)) {
        $order_statuses = false;
        $data['order_statuses'] = '';
    }

    if (!$new_order && !$new_customer && !$order_statuses) {
        Db::getInstance()->delete('mobassistantconnector_push_notifications', '`user_id` = '.(int)$user_id);
        deleteEmptyDevices();
        deleteEmptyAccounts();
    } else {
        if (!empty($data)) {
            Db::getInstance()->update('mobassistantconnector_push_notifications', $data, '`user_id` = '.(int)$user_id);
        }
    }
}

function getUserActions($user_id)
{
    $user_actions = Db::getInstance()->getValue(
        'SELECT `allowed_actions` FROM `'._DB_PREFIX_ . 'mobassistantconnector_users` WHERE `user_id` = '.$user_id
    );

    if ($user_actions !== false) {
        $result = getUserActionsFromCodes($user_actions);
    } else {
        $result = 'error';
    }

    return $result;
}

function getActionsByCodes($actions_codes)
{
    $restricted_actions = getRestrictedActions();
    $actions = array();

    if (!empty($actions_codes)) {
        $actions = explode(';', $actions_codes);
    }

    return getActionsAsArray($restricted_actions, $actions);
}

function checkUserData($login, $user_id)
{
    $is_login_exists = Db::getInstance()->getValue(
        'SELECT `user_id` FROM `'._DB_PREFIX_."mobassistantconnector_users` WHERE `username` = '".$login
        ."' AND `user_id` <> ".$user_id
    );

    if (!$is_login_exists) {
        return 'success';
    } else {
        return 'The same login already exists';
    }
}

function saveUser($user_id, $login, $password, $actions_codes, $value, $is_new_user)
{
    if ($is_new_user == '1') {
        $result = Db::getInstance()->insert(
            'mobassistantconnector_users',
            array(
                'username' => $login,
                'password' => md5($password),
                'allowed_actions' => $actions_codes,
                'status' => (int)$value,
                'qr_code_hash' => hash('sha256', time())
            )
        );
    } else {
        $current_password = Db::getInstance()->getValue(
            'SELECT `password` FROM `'._DB_PREFIX_.'mobassistantconnector_users` WHERE `user_id` = '.(int)$user_id
        );

        if ($current_password != $password) {
            $password = md5($password);
        }

        $result = Db::getInstance()->update(
            'mobassistantconnector_users',
            array('username' => $login, 'password' => $password),
            '`user_id` = '.(int)$user_id
        );
        Db::getInstance()->delete('mobassistantconnector_session_keys', '`user_id` = '.(int)$user_id);
    }

    if ($result) {
        return 'success';
    }

    return 'error';
}

function getUserByUsername($login)
{
    $result = array();
    $user_obj = new DbQuery();
    $user_obj->select(
        'mu.`user_id`,
        mu.`username`,
        mu.`password`,
        mu.`allowed_actions`,
        mu.`qr_code_hash`,
        mu.`status`'
    );
    $user_obj->from('mobassistantconnector_users', 'mu');
    $user_obj->where("mu.`username` = '".$login."'");
    $user_sql = $user_obj->build();
    $user = Db::getInstance()->executeS($user_sql);

    if (isset($user[0])) {
        $user[0]['allowed_actions'] = getUserActionsFromCodes($user[0]['allowed_actions']);
        $user[0]['qr_code_data'] = call_user_func(
            'base64_encode',
            Tools::jsonEncode(
                array('url' => getShopUrl(), 'login' => $user[0]['username'], 'password' => $user[0]['password'])
            )
        );
        $user[0]['pushes'] = getUserPushes($user[0]['user_id']);
        $result = $user[0];
    }

    return $result;
}

function regenerateQrCodeHash($user_id)
{
    $hash = hash('sha256', time());
    $result = Db::getInstance()->update(
        'mobassistantconnector_users',
        array('qr_code_hash' => $hash),
        '`user_id` = '.$user_id
    );

    if ($result) {
        return array('hash' => $hash);
    }

    return 'error';
}

function formDevices($devices)
{
    $count_devices  = count($devices);
    $devices_output = array();
    $statuses = getOrderStatuses();

    for ($i = 0; $i < $count_devices; $i++) {
        $device_unique = !$devices[$i]['device_unique_id'] ? 'Unknown' : $devices[$i]['device_unique_id'];
        $order_statuses = '-';

        if ($devices[$i]['id_shop'] > 0) {
            $shop = new Shop($devices[$i]['id_shop']);
            $devices[$i]['shop'] = $shop->name;
        } else {
            $devices[$i]['shop'] = 'All';
        }

        if ($devices[$i]['last_activity'] == '0000-00-00 00:00:00') {
            $devices[$i]['last_activity'] = '';
        }

        if ($device_unique == 'Unknown') {
            $devices[$i]['device_name'] = 'Unknown';
        }

        if ($devices[$i]['order_statuses'] == '-1') {
            $order_statuses = 'All';
        } elseif (!empty($devices[$i]['order_statuses'])) {
            $push_order_statuses = explode('|', $devices[$i]['order_statuses']);
            $order_statuses = array();
            $all = true;

            foreach ($statuses as $id => $name) {
                if (in_array($id, $push_order_statuses)) {
                    $order_statuses[] = $name;
                } else {
                    $all = false;
                }
            }

            if ($all) {
                $order_statuses = 'All';
            } elseif (!empty($order_statuses)) {
                $order_statuses = implode(', ', $order_statuses);
            }
        }

        $devices_output[$device_unique]['device_name'] = !$devices[$i]['device_name']
            ? '-'
            : $devices[$i]['device_name'];
        $devices_output[$device_unique]['account_email'] = !$devices[$i]['account_email']
            ? '-'
            : $devices[$i]['account_email'];
        $devices_output[$device_unique]['last_activity'] = !$devices[$i]['last_activity']
            ? '-'
            : $devices[$i]['last_activity'];
        $devices_output[$device_unique]['status'] = $devices[$i]['status'];
        $devices_output[$device_unique]['account_id'] = $devices[$i]['account_id'];
        $devices_output[$device_unique]['pushes'][] = array(
            'id'                => $devices[$i]['id'],
            'shop'              => $devices[$i]['shop'],
            'app_connection_id' => $devices[$i]['app_connection_id'],
            'new_order'         => $devices[$i]['new_order'],
            'new_customer'      => $devices[$i]['new_customer'],
            'order_statuses'    => $order_statuses,
            'currency_iso'      => !$devices[$i]['currency_iso'] ? '-' : $devices[$i]['currency_iso'],
        );
    }

    return $devices_output;
}

function getOrderStatuses()
{
    $result = array();
    $statuses = OrderState::getOrderStates(Configuration::get('PS_LANG_DEFAULT'));
    $count = count($statuses);

    for ($i = 0; $i < $count; $i++) {
        $result[$statuses[$i]['id_order_state']] = $statuses[$i]['name'];
    }

    return $result;
}

function replaceNull($data)
{
    if (!is_array($data)) {
        $data = array();
    }

    foreach ($data as $index => $values) {
        foreach ($values as $key => $value) {
            if ($value === null) {
                $data[$index][$key] = '';
            }
        }
    }

    return $data;
}

function prepareIds($data)
{
    if (!$data) {
        return false;
    }

    $ids   = array();
    $arr   = explode(',', $data);
    $count = count($arr);

    for ($i = 0; $i < $count; $i++) {
        $ids[] = (int)trim($arr[$i]);
    }

    return implode(',', $ids);
}

function getUserActionsFromCodes($codes = '')
{
    $restricted_actions = getRestrictedActions();

    if (empty($codes)) {
        $result = 'Nothing';
    } else {
        $action_codes = explode(';', $codes);
        $result       = getUserAllowedActionsAsString($action_codes, $restricted_actions);
    }

    return $result;
}

function getActionsAsArray($restricted_actions, $actions_codes)
{
    $groups_count = count($restricted_actions);

    for ($i = 0; $i < $groups_count; $i++) {
        $child_count = count($restricted_actions[$i]['child']);

        for ($j = 0; $j < $child_count; $j++) {
            if (!empty($actions_codes) && in_array($restricted_actions[$i]['child'][$j]['code'], $actions_codes)) {
                $restricted_actions[$i]['child'][$j]['allowed'] = 1;
            } else {
                $restricted_actions[$i]['child'][$j]['allowed'] = 0;
            }
        }
    }

    return $restricted_actions;
}

function getUserAllowedActionsAsString($action_codes, $restricted_actions)
{
    $is_allowed_all = true;
    $result = '';
    $groups_count = count($restricted_actions);

    for ($i = 0; $i < $groups_count; $i++) {
        $child_count = count($restricted_actions[$i]['child']);
        $tmp = '';

        for ($j = 0; $j < $child_count; $j++) {
            if (in_array($restricted_actions[$i]['child'][$j]['code'], $action_codes)) {
                if (!empty($tmp)) {
                    $tmp .= ', ';
                }

                $tmp .= $restricted_actions[$i]['child'][$j]['name'];
            } else {
                $is_allowed_all = false;
            }
        }

        if (!empty($tmp)) {
            if (!empty($result)) {
                $result .= ', ';
            }

            $result .= $restricted_actions[$i]['group_name'].' ('.$tmp.')';
        }
    }

    if ($is_allowed_all) {
        $result = 'All';
    }

    return $result;
}

function getShopUrl()
{
    $shop_info = new Shop((int)Configuration::get('PS_SHOP_DEFAULT'));
    $shop_url = $shop_info->getBaseURL();
    $shop_url = str_replace('http://', '', $shop_url);
    $shop_url = str_replace('https://', '', $shop_url);
    preg_replace('/\/*$/i', '', $shop_url);

    return $shop_url;
}
