/**
 *    This file is part of Mobile Assistant Connector.
 *
 *   Mobile Assistant Connector is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   Mobile Assistant Connector is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Mobile Assistant Connector.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  @author    eMagicOne <contact@emagicone.com>
 *  @copyright 2014-2018 eMagicOne
 *  @license   http://www.gnu.org/licenses   GNU General Public License
 */

$(document).ready(function() {
    var baseUrl                = $('#mobassistantconnector_base_url').val();
    var adminModuleUrl         = $('#mobassistantconnector_admin_module_url').val();
    var loader_img             = '<img src="' + baseUrl + 'modules/mobassistantconnector/views/img/loader.gif">';
    var loader                 = '<div class="mobassistantconnector_loader">' + loader_img + '</div>';
    var loader_users_container = '<div id="mobassistantconnector_users_loader_container">' + loader + '</div>';
    var users_parent           = $('#mobassistantconnector_users').parent();
    var key                    = $('#mobassistantconnector_key').val();
    var new_users_count        = 0;
    var password = $('#mobassistantconnector_password');
    var login    = $('#mobassistantconnector_login');
    var yes      = '<img src="' + baseUrl + 'modules/mobassistantconnector/views/img/yes.png">';
    var no       = '<img src="' + baseUrl + 'modules/mobassistantconnector/views/img/no.png">';
    var enabled  = '<img src="' + baseUrl + 'modules/mobassistantconnector/views/img/enabled.gif" title="Disable">';
    var disabled = '<img src="' + baseUrl + 'modules/mobassistantconnector/views/img/disabled.gif" title="Enable">';

    var mobassistantconnector_users_container = '<div id="mobassistantconnector_users_container">' +
        loader_users_container + '</div>';
    var mobassistantconnector_users_table_div = '<div id="mobassistantconnector_users_table_div"></div>';

    users_parent.append(mobassistantconnector_users_container);
    $('#mobassistantconnector_users_container').append(mobassistantconnector_users_table_div);

    getUsers();
    getUnlinkedPushes();
    setEvents();
    generateAppQRCode();

    function generateAppQRCode() {
        $("#mobassistantconnector_qrcode_app").parent().prepend(
            "<div id='mobassistantconnector_qr_code_app_img' style='padding-bottom: 10px'></div>"
        );
        var qrcodeAppContainer = document.getElementById("mobassistantconnector_qr_code_app_img");

        if (qrcodeAppContainer != null) {
            var qrCode = new QRCode(qrcodeAppContainer, {
                width : 150,
                height : 150
            });
            qrCode.makeCode("https://goo.gl/bL3nEi");
        }
    }

    function setEvents() {
        $('div.mobassistantconnector_status').live('click', function() {
            changeStatus($(this));
        });

        $('div.mobassistantconnector_pushes_delete').live('click', function() {
            if (confirm('Are you sure you want to delete push notification data?')) {
                deletePush($(this).attr('push_id'), $(this).attr('user_id'));
            }
        });

        $('div.mobassistantconnector_users_delete').live('click', function() {
            if (isNewUser($(this)) == '0') {
                if (confirm('Are you sure you want to delete user?')) {
                    deleteUser($(this).attr('user_id'));
                }
            } else {
                var user_id = $(this).attr('user_id');

                $('#mobassistantconnector_row_user_' + user_id).remove();
                $('#mobassistantconnector_row_loader_user_' + user_id).remove();
            }
        });

        $('#mobassistantconnector_users_bulk_select_all').live('click', function() {
            $('.mobassistantconnector_users').each(function() {
                $(this).prop('checked', true);
            });
        });

        $('#mobassistantconnector_users_bulk_unselect_all').live('click', function() {
            $('.mobassistantconnector_users').each(function() {
                $(this).prop('checked', false);
            });
        });

        $('.mobassistantconnector_users_td_actions').live('click', function() {
            var user_id = $(this).attr('user_id');

            showUserPermissionsDialog(user_id);

            if (isNewUser($(this)) == '0') {
                getActions(user_id);
            } else {
                getActionsForNewUser(user_id);
            }
        });

        $('.mobassistantconnector_users_icon_qr_code').live('click', function() {
            var user_id = $(this).attr('user_id');
            var qr_code_link = $('#mobassistantconnector_qr_code_link_user_' + user_id).val();

            $('#mobassistantconnector_qr_code_scan_user').html('');

            var qrcode_container = document.getElementById('mobassistantconnector_qr_code_scan_user');
            var qrcode = new QRCode(qrcode_container, {
                width : 250,
                height : 250
            });

            qrcode.makeCode($(this).attr('qr_code_data'));
            showUserQrCodeDialog(user_id);

            $('#mobassistantconnector_qr_code_link_user').html('<a href="' + qr_code_link + '" target="_blank">' +
                qr_code_link + '</a>');
            $('#mobassistantconnector_qr_code_user').show();
            setDialogPosition($('#mobassistantconnector_users_qr_code_dialog'));
        });

        $('#mobassistantconnector_user_permissions_toggle_all').live('click', function() {
            var is_checked = $('#mobassistantconnector_user_permissions_chb_toggle_all').is(':checked');

            $('.mobassistantconnector_user_permissions').each(function() {
                $(this).prop('checked', is_checked);
            });
        });

        $('#mobassistantconnector_add_user').live('click', function() {
            addNewUserRow();
        });

        $('.mobassistantconnector_users_save').live('click', function() {
            checkUserData($(this).attr('user_id'), isNewUser($(this)));
        });

        $('.mobassistantconnector_users_status').live('click', function() {
            if (isNewUser($(this)) == '0') {
                changeStatusForUser($(this));
            } else {
                changeStatusForNewUser($(this));
            }
        });

        $('#mobassistantconnector_users_bulk_status_active').live('click', function() {
            changeUserStatusBulk(1);
        });

        $('#mobassistantconnector_users_bulk_status_inactive').live('click', function() {
            changeUserStatusBulk(0);
        });

        $('#mobassistantconnector_users_bulk_delete').live('click', function() {
            if (confirm('Are you sure you want to delete selected users?')) {
                deleteUserBulk();
            }
        });

        $('.mobassistantconnector_users_login').live('click', function() {
            $(this).keyup(function() {
                processSaveButtonVisibility($(this).attr('user_id'));
            });
        });

        $('.mobassistantconnector_users_password').live('click', function() {
            $(this).keyup(function() {
                processSaveButtonVisibility($(this).attr('user_id'));
            });
        });

        $('.mobassistantconnector_users_save_login_password').live('click', function() {
            checkUserData($(this).attr('user_id'), isNewUser($(this)));
        });

        $('.mobassistantconnector_user_permissions').live('change', function() {
            processSelectedAction($(this));
        });

        $('.mobassistantconnector_users_td_actions_text').live('mouseenter', function() {
            $('#mobassistantconnector_users_actions_text_user_' + $(this).attr('user_id'))
                .addClass('mobassistantconnector_actions_hover');
        }).live('mouseleave', function() {
                $('#mobassistantconnector_users_actions_text_user_' + $(this).attr('user_id'))
                    .removeClass('mobassistantconnector_actions_hover');
            });

        $('.mobassistantconnector_btn_users_devices').live('click', function() {
            var user_id = $(this).attr('user_id');
            var pushes = $('#mobassistantconnector_row_pushes_user_' + user_id);
            var button = $('#mobassistantconnector_btn_devices_user_' + user_id);

            if (pushes.is(':visible')) {
                pushes.hide();
                button.html('Show devices');
            } else {
                pushes.show();
                button.html('Hide devices');
            }
        });

        $(document).on('click', function(e) {
            var bulk_users_list = $('#mobassistantconnector_users_bulk_list');

            if (!bulk_users_list.is(':visible') && e.target.hasAttribute('id')
                && (e.target.id == 'mobassistantconnector_users_bulk_btn' || e.target.id == 'mobassistantconnector_users_bulk_btn_img')) {
                bulk_users_list.show();
            } else if (bulk_users_list.is(':visible')) {
                bulk_users_list.hide();
            }
        });
    }

    function changeStatus(selector) {
        var val      = selector.attr('val');
        var img_prev = selector.html();

        selector.html(loader_img);

        if (val == 0) {
            val = 1;
        } else {
            val = 0;
        }

        //$.post(baseUrl + 'modules/mobassistantconnector/functions/ajax.php',
        $.post(adminModuleUrl,
            {
                call_function: 'changeStatus',
                account_id: selector.attr('account_id'),
                value: val,
                auth_key: key
            },
            function(data) {
                if (data == 'success') {
                    var enabled = '<img src="' + baseUrl + 'modules/mobassistantconnector/views/img/enabled.gif" ' +
                        'title="Disable ' + selector.attr("account_email") + '">';
                    var disabled = '<img src="' + baseUrl + 'modules/mobassistantconnector/views/img/disabled.gif" ' +
                        'title="Enable ' + selector.attr("account_email") + '">';

                    if (val == 1) {
                        selector.html(enabled);
                        selector.attr('val', 1);
                    } else {
                        selector.html(disabled);
                        selector.attr('val', 0)
                    }
                } else {
                    if (data == "error") {
                        data = "Some error occurred while changing account status";
                    }

                    selector.html(img_prev);
                    alert(data);
                }
            }, 'json'
        ).error(function(error) {
                selector.html(img_prev);
                alert("Mobile Assistant Connector Error:\r\n" + error.responseText);
            });
    }

    function changeUserStatusBulk(val) {
        var user_ids = getUserIds();

        if (user_ids.length == 0) {
            alert('Nothing selected');
            return;
        }

        var loader = $('#mobassistantconnector_users_loader_container');

        loader.show();
        $('#mobassistantconnector_users_table_div').addClass('mobassistantconnector_inactive');

        //$.post(baseUrl + 'modules/mobassistantconnector/functions/ajax.php',
        $.post(adminModuleUrl,
            {
                call_function: 'changeStatusForUser',
                user_ids: user_ids,
                value: val,
                auth_key: key
            },
            function(data) {
                if (data != 'success') {
                    alert(data);
                }

                getUsers();
            }, 'json'
        ).error(function(error) {
                loader.hide();
                alert("Mobile Assistant Connector Error:\r\n" + error.responseText);
            });
    }

    function changeStatusForUser(selector) {
        var val      = selector.attr('val');
        var img_prev = selector.html();

        selector.html(loader_img);

        if (val == 0) {
            val = 1;
        } else {
            val = 0;
        }

        //$.post(baseUrl + 'modules/mobassistantconnector/functions/ajax.php',
        $.post(adminModuleUrl,
            {
                call_function: 'changeStatusForUser',
                user_ids: selector.attr('user_id'),
                value: val,
                auth_key: key
            },
            function(data) {
                if (data == 'success') {
                    if (val == 1) {
                        selector.html(enabled);
                        selector.attr('val', 1);
                    } else {
                        selector.html(disabled);
                        selector.attr('val', 0)
                    }
                } else {
                    selector.html(img_prev);
                    alert(data);
                }
            }, 'json'
        ).error(function(error) {
                selector.html(img_prev);
                alert("Mobile Assistant Connector Error:\r\n" + error.responseText);
            });
    }

    function changeStatusForNewUser(element) {
        if (element.attr('val') == '1') {
            element.html(disabled);
            element.attr('val', '0');
        } else {
            element.html(enabled);
            element.attr('val', '1');
        }
    }

    function deleteUserBulk() {
        var user_ids = getUserIds();

        if (user_ids.length == 0) {
            alert('Nothing selected');
            return;
        }

        var loader_bulk = $('#mobassistantconnector_users_loader_container');

        loader_bulk.show();
        $('#mobassistantconnector_users_table_div').addClass('mobassistantconnector_inactive');

        //$.post(baseUrl + 'modules/mobassistantconnector/functions/ajax.php',
        $.post(adminModuleUrl,
            {
                call_function: 'deleteUser',
                user_ids: user_ids,
                auth_key: key
            },
            function(data) {
                if (data != 'success') {
                    alert(data);
                }

                getUsers();
            }, 'json'
        ).error(function(error) {
                loader_bulk.hide();
                alert("Mobile Assistant Connector Error:\r\n" + error.responseText);
            });
    }

    function deleteUser(user_id) {
        var row_user = $('#mobassistantconnector_row_user_' + user_id);
        var row_user_pushes = $('#mobassistantconnector_row_pushes_user_' + user_id);
        var is_pushes_visible = row_user_pushes.is(':visible');
        var row_user_loader = $('#mobassistantconnector_row_loader_user_' + user_id);

        row_user.hide();
        row_user_pushes.hide();
        row_user_loader.show();

        //$.post(baseUrl + 'modules/mobassistantconnector/functions/ajax.php',
        $.post(adminModuleUrl,
            {
                call_function: 'deleteUser',
                user_ids: user_id,
                auth_key: key
            },
            function(data) {
                if (data == 'success') {
                    row_user.remove();
                    row_user_pushes.remove();
                    row_user_loader.remove();
                } else {
                    row_user_loader.hide();
                    row_user.show();

                    if (is_pushes_visible) {
                        row_user_pushes.show();
                    }

                    alert(data);
                }
            }, 'json'
        ).error(function(error) {
                row_user_loader.hide();
                row_user.show();

                if (is_pushes_visible) {
                    row_user_pushes.show();
                }

                alert("Mobile Assistant Connector Error:\r\n" + error.responseText);
            });
    }

    function deletePush(push_id, user_id) {
        var pushes_table = $('#mobassistantconnector_table_pushes_user_' + user_id);
        var pushes_loader = $('#mobassistantconnector_pushes_loader_user_' + user_id);

        pushes_table.hide();
        pushes_loader.show();

        //$.post(baseUrl + 'modules/mobassistantconnector/functions/ajax.php',
        $.post(adminModuleUrl,
            {
                call_function: 'deletePush',
                push_id: push_id,
                auth_key: key
            },
            function(data) {
                if (data == 'success') {
                    getUserPushes(user_id, pushes_table, pushes_loader);
                } else {
                    pushes_loader.hide();
                    pushes_table.show();
                    alert(data);
                }
            }, 'json'
        ).error(function(error) {
                pushes_loader.hide();
                pushes_table.show();
                alert("Mobile Assistant Connector Error:\r\n" + error.responseText);
            });
    }

    function getUserPushes(user_id, pushes_table, pushes_loader) {
        //$.post(baseUrl + 'modules/mobassistantconnector/functions/ajax.php',
        $.post(adminModuleUrl,
            {
                call_function: 'getUserPushes',
                user_id: user_id,
                auth_key: key
            },
            function(data) {
                if (data != 'error' && data != 'Authentication error') {
                    $('#mobassistantconnector_pushes_container_user_' + user_id).html(generateUserRowPushes(data, user_id));
                } else {
                    if (data == "error") {
                        data = "Some error occurred while getting user pushes";
                    }

                    pushes_loader.hide();
                    pushes_table.show();
                    alert(data);
                }
            }, 'json'
        ).error(function(error) {
                pushes_loader.hide();
                pushes_table.show();
                alert("Mobile Assistant Connector Error:\r\n" + error.responseText);
            });
    }

    function getUnlinkedPushes() {
        var pushes_unlinked = $('#mobassistantconnector_unlinked_pushes');

        if (pushes_unlinked.length == 0) {
            return;
        }

        pushes_unlinked
            .parent()
            .prepend('<div id="mobassistantconnector_row_pushes_user_-1"></div>');
        $('#mobassistantconnector_row_pushes_user_-1').html(
            generateUserRowPushes($.parseJSON(pushes_unlinked.val()), '-1')
        );
    }

    function getUserIds() {
        var user_ids = '';

        $('[name="mobassistantconnector_users[]"]:checked').each(function() {
            if (user_ids != '') {
                user_ids += ',' + $(this).val();
            } else {
                user_ids = $(this).val();
            }
        });

        return user_ids;
    }

    function getSelectedActions() {
        var selected_actions = '';

        $('.mobassistantconnector_user_permissions:checked').each(function() {
            if (selected_actions.length > 0) {
                selected_actions += ';' + $(this).val();
            } else {
                selected_actions = $(this).val();
            }
        });

        return selected_actions;
    }

    function showSelectedActionsForNewUser(user_id) {
        var selected_actions       = '';
        var selected_actions_codes = '';
        var not_checked            = $('.mobassistantconnector_user_permissions:not(:checked)');
        var checked                = $('.mobassistantconnector_user_permissions:checked');

        if (not_checked.length > 0) {
            if (checked.length > 0) {
                checked.each(function() {
                    if (selected_actions != '') {
                        selected_actions += ', ';
                    }

                    if (selected_actions_codes != '') {
                        selected_actions_codes += ';';
                    }

                    selected_actions += $(this).parent().text().trim();
                    selected_actions_codes += $(this).val();
                });
            } else {
                selected_actions = 'Nothing';
            }
        } else {
            checked.each(function() {
                if (selected_actions_codes != '') {
                    selected_actions_codes += ';';
                }

                selected_actions_codes += $(this).val();
            });

            selected_actions = 'All';
        }

        $('#mobassistantconnector_div_actions_text_user_' + user_id).html(selected_actions);
        $('#mobassistantconnector_actions_codes_user_' + user_id).val(selected_actions_codes);
        preparePermissionsToView(true, user_id);
        $('#mobassistantconnector_users_permissions_dialog').dialog('close');
    }

    function getUsers() {
        var loader_users_bulk = $('#mobassistantconnector_users_loader_container');

        //$.post(baseUrl + 'modules/mobassistantconnector/functions/ajax.php',
        $.post(adminModuleUrl,
            {
                call_function: 'getUsers',
                auth_key: key
            },
            function (data) {
                var target = $('#mobassistantconnector_users_table_div');

                if (data != 'error' && data != 'Authentication error') {
                    var status;
                    var table_data = '<table id="mobassistantconnector_users_table" class="table product"><tr><th></th>' +
                        '<th style="width: 30%"><table width="100%"><tr><th style="width: 50%">Login</th>' +
                        '<th style="width: 50%">Password</th></tr></table>' +
                        '<th id="mobassistantconnector_users_permissions">Permissions</th><th>QR-code</th>' +
                        '<th>Status</th><th></th><th></th><th></th></tr>';
                    var count_users = data.length;

                    for (var i = 0; i < count_users; i++) {
                        table_data += '<tr id="mobassistantconnector_row_user_' + data[i].user_id + '">' +
                            generateUserRow(data[i]) + '</tr><tr class="mobassistantconnector_users_row_pushes" ' +
                            'id="mobassistantconnector_row_pushes_user_' + data[i].user_id + '"><td colspan="8" ' +
                            'class="mobassistantconnector_users_td_pushes" id="mobassistantconnector_pushes_container_user_' +
                            data[i].user_id + '">' +
                            generateUserRowPushes(data[i].pushes, data[i].user_id) + '</td></tr>' +
                            generateUserRowLoader(data[i].user_id);
                    }

                    if (count_users == 0) {
                        table_data += '<tr class="mobassistantconnector_users_no_data"><td colspan="6" ' +
                            'style="text-align: center">No data</td></tr>';
                    }

                    table_data += '</table><table><tr><td><div ' +
                        'style="margin-top: 10px; margin-bottom: 10px; margin-right: 15px"><button type="button" ' +
                        'id="mobassistantconnector_users_bulk_btn" class="btn btn-default">Bulk actions ' +
                        '<img id="mobassistantconnector_users_bulk_btn_img" src="' + baseUrl +
                        'modules/mobassistantconnector/views/img/down.png"></button></div>' +
                        '<div id="mobassistantconnector_users_bulk_list">' +
                        '<table><tr id="mobassistantconnector_users_bulk_select_all"><td><img src="' + baseUrl +
                        'modules/mobassistantconnector/views/img/checked.png"></td><td>Select all</td></tr>' +
                        '<tr id="mobassistantconnector_users_bulk_unselect_all"><td><img src="' + baseUrl +
                        'modules/mobassistantconnector/views/img/unchecked.png"></td><td>Unselect all</td>' +
                        '</tr><tr class="mobassistantconnector_delimiter"><td colspan="2"><hr></td></tr>' +
                        '<tr id="mobassistantconnector_users_bulk_status_active"><td><img width="12px" height="12px" ' +
                        'src="' + baseUrl + 'modules/mobassistantconnector/views/img/enabled.gif"></td>' +
                        '<td>Change status to active</td>' + '</tr>' +
                        '<tr id="mobassistantconnector_users_bulk_status_inactive"><td><img width="12px" height="12px" ' +
                        'src="' + baseUrl + 'modules/mobassistantconnector/views/img/disabled.gif"></td>' +
                        '<td>Change status to inactive</td></tr>' +
                        '<tr class="mobassistantconnector_delimiter"><td colspan="2"><hr></td></tr>' +
                        '<tr id="mobassistantconnector_users_bulk_delete"><td><img src="' + baseUrl +
                        'modules/mobassistantconnector/views/img/bulk_trash.png"></td><td>Delete selected</td></tr>' +
                        '</table></div></td><td><div>' + '<button type="button" id="mobassistantconnector_add_user" ' +
                        'class="btn btn-default pull-right">Add user</button></div></td></tr></table>' +
                        '<div id="mobassistantconnector_users_permissions_dialog" ' +
                        'title="User Permissions">' + getUserPermissionsDialogBody() + '</div><div ' +
                        'id="mobassistantconnector_users_qr_code_dialog" title="QR-code for PrestaShop Mobile Assistant App">' +
                        getUserQrCodeDialogBody() + '</div>';
                    loader_users_bulk.hide();
                    target.removeClass('mobassistantconnector_inactive');
                    target.html(table_data);

                    preparePermissionsToView(false, 0);
                } else {
                    if (data == "error") {
                        data = "Some error occurred while getting users";
                    }

                    loader_users_bulk.hide();
                    alert(data);
                }
            }, 'json'
        ).error(function(error) {
                loader_users_bulk.hide();
                alert("Mobile Assistant Connector Error:\r\n" + error.responseText);
            });
    }

    /**
     * Add some style to view users' permissions
     * If is_new_user = false then user_id is ignored else user_id must be equal new user id
     * @param is_new_user
     * @param user_id
     */
    function preparePermissionsToView(is_new_user, user_id) {
        var width;
        var elements;

        if (!is_new_user) {
            elements = $('.mobassistantconnector_div_user_actions_text');
            width = $('#mobassistantconnector_users_permissions').width() - 44
        } else {
            elements = $('#mobassistantconnector_div_actions_text_user_' + user_id);
            width = $('.mobassistantconnector_div_user_actions_text:first').width();
        }

        $.each(elements, function() {
            $(this).css('width', width + 'px').addClass('mobassistantconnector_div_user_actions_text_view');
        });
    }

    function getActions(user_id) {
        var actions_container = $('#mobassistantconnector_user_actions_container');
        var loader_actions = $('#mobassistantconnector_loader_actions');

        actions_container.removeClass('mobassistantconnector_inactive');
        actions_container.html('');
        loader_actions.css({paddingTop: '14px'});
        loader_actions.show();

        //$.post(baseUrl + 'modules/mobassistantconnector/functions/ajax.php',
        $.post(adminModuleUrl,
            {
                call_function: 'getActions',
                user_id: user_id,
                auth_key: key
            },
            function (data) {
                if (data != 'error' && data != 'Authentication error') {
                    var actions_data = generateUserPermissionsDialogBodyFromData(data);

                    loader_actions.hide();
                    loader_actions.css({paddingTop: '0'});
                    actions_container.html(actions_data);
                    setDialogPosition($('#mobassistantconnector_users_permissions_dialog'));
                } else {
                    if (data == "error") {
                        data = "Some error occurred while getting user actions";
                    }

                    loader_actions.hide();
                    alert(data);
                }
            }, 'json'
        ).error(function(error) {
                loader_actions.hide();
                alert("Mobile Assistant Connector Error:\r\n" + error.responseText);
            });
    }

    function generateUserPermissionsGroupName(group_name) {
        return '<div class="mobassistantconnector_user_permissions_group">' + group_name + '</div>';
    }

    function generateUserPermissionsRow(permission_data) {
        var checked = '';

        if (permission_data.allowed == 1) {
            checked = 'checked';
        }

        return '<div class="mobassistantconnector_user_permission">' +
            '<label class="mobassistantconnector_user_permission_label"><input type="checkbox" ' +
            'id="mobassistantconnector_user_action_' + permission_data.code +
            '" class="mobassistantconnector_user_permissions" value="' + permission_data.code +
            '" ' + checked + '> ' + permission_data.name + '</label></div>';
    }

    function getActionsForNewUser(user_id) {
        var actions_container = $('#mobassistantconnector_user_actions_container');
        var loader_actions    = $('#mobassistantconnector_loader_actions');

        actions_container.removeClass('mobassistantconnector_inactive');
        actions_container.html('');
        loader_actions.css({paddingTop: '14px'});
        loader_actions.show();

        //$.post(baseUrl + 'modules/mobassistantconnector/functions/ajax.php',
        $.post(adminModuleUrl,
            {
                call_function: 'getActionsByCodes',
                user_id: user_id,
                actions_codes: $('#mobassistantconnector_actions_codes_user_' + user_id).val(),
                auth_key: key
            },
            function (data) {
                if (data != 'error' && data != 'Authentication error') {
                    var actions_data = generateUserPermissionsDialogBodyFromData(data);

                    loader_actions.hide();
                    loader_actions.css({paddingTop: '0'});
                    actions_container.html(actions_data);
                    setDialogPosition($('#mobassistantconnector_users_permissions_dialog'));
                } else {
                    if (data == "error") {
                        data = "Some error occurred while getting user actions";
                    }

                    loader_actions.hide();
                    alert(data);
                }
            }, 'json'
        );
    }

    function generateUserPermissionsDialogBodyFromData(data) {
        var actions_data = '';
        var groups_count = data.length;
        var child_count;
        var checked_all = 'checked';

        for (var i = 0; i < groups_count; i++) {
            child_count = data[i].child.length;
            actions_data += generateUserPermissionsGroupName(data[i].group_name);

            for (var j = 0; j < child_count; j++) {
                actions_data += generateUserPermissionsRow(data[i].child[j]);

                if (data[i].child[j].allowed == 0) {
                    checked_all = '';
                }
            }
        }

        if (groups_count == 0) {
            actions_data = '<div class="mobassistantconnector_no_data">No data</div>';
        } else {
            actions_data = '<div class="mobassistantconnector_user_permissions_toggle_all">' +
                '<label id="mobassistantconnector_user_permissions_toggle_all"><input type="checkbox" ' +
                ' id="mobassistantconnector_user_permissions_chb_toggle_all" value="1" ' + checked_all +
                '>Toggle all</label></div>' + actions_data;
        }

        return actions_data;
    }

    function saveUserActions(user_id) {
        var selected_actions  = getSelectedActions();
        var loader_actions    = $('#mobassistantconnector_loader_actions');
        var actions_container = $('#mobassistantconnector_user_actions_container');

        loader_actions.show();
        actions_container.addClass('mobassistantconnector_inactive');

        //$.post(baseUrl + 'modules/mobassistantconnector/functions/ajax.php',
        $.post(adminModuleUrl,
            {
                call_function: 'saveUserActions',
                selected_actions: selected_actions,
                user_id: user_id,
                auth_key: key
            },
            function (data) {
                $('#mobassistantconnector_users_actions_user_' + user_id).hide();

                if (data == 'success') {
                    $('#mobassistantconnector_users_permissions_dialog').dialog('close');
                    updateAllowedActionsText(user_id);
                } else {
                    if (data == "error") {
                        data = "Some error occurred while saving user actions";
                    }

                    loader_actions.hide();
                    actions_container.removeClass('mobassistantconnector_inactive');
                    alert(data);
                }
            }, 'json'
        ).error(function() {
                loader_actions.hide();
                actions_container.removeClass('mobassistantconnector_inactive');
                alert('Some error occurred while saving user actions');
            });
    }

    function updateAllowedActionsText(user_id) {
        var actions_update_loader = $('#mobassistantconnector_actions_text_loader_user_' + user_id);
        var actions_text = $('#mobassistantconnector_div_actions_text_user_' + user_id);

        actions_update_loader.show();
        actions_text.addClass('mobassistantconnector_inactive');

        //$.post(baseUrl + 'modules/mobassistantconnector/functions/ajax.php',
        $.post(adminModuleUrl,
            {
                call_function: 'getUserActions',
                user_id: user_id,
                auth_key: key
            },
            function (data) {
                actions_update_loader.hide();
                actions_text.removeClass('mobassistantconnector_inactive');

                if (data != 'error' && data != 'Authentication error') {
                    actions_text.html(data);
                } else {
                    if (data == "error") {
                        data = "Some error occurred while updating allowed actions text";
                    }

                    alert(data);
                }
            }, 'json'
        ).error(function(error) {
                alert("Mobile Assistant Connector Error:\r\n" + error.responseText);
            });
    }

    function addNewUserRow() {
        var data = {};

        new_users_count++;
        data.user_id = 'new_' + new_users_count;
        data.status = 1;
        data.username = '';
        data.password = '';
        data.allowed_actions = 'Nothing';
        $('#mobassistantconnector_users_table > tbody:last-child').append(
            '<tr id="mobassistantconnector_row_user_' + data.user_id + '">' + generateUserRow(data) + '</tr>' +
            generateUserRowLoader(data.user_id)
        );
    }

    function checkUserData(user_id, is_new_user) {
        var login    = $('#mobassistantconnector_login_user_' + user_id).val().trim();
        var password = $('#mobassistantconnector_password_user_' + user_id).val();

        if (login == '' || password.trim() == '') {
            alert('Check if login or password are not empty!');
            return;
        }

        var row_user        = $('#mobassistantconnector_row_user_' + user_id);
        var row_user_loader = $('#mobassistantconnector_row_loader_user_' + user_id);

        row_user.hide();
        row_user_loader.show();

        //$.post(baseUrl + 'modules/mobassistantconnector/functions/ajax.php',
        $.post(adminModuleUrl,
            {
                call_function: 'checkUserData',
                login: login,
                user_id: user_id,
                auth_key: key
            },
            function (data) {
                if (data == 'success') {
                    saveUser(user_id, is_new_user, login, password);
                } else {
                    if (data == "error") {
                        data = "Some error occurred while checking user data";
                    }

                    row_user_loader.hide();
                    row_user.show();
                    alert(data);
                }
            }, 'json'
        ).error(function() {
                row_user_loader.hide();
                row_user.show();
                alert('Some error occurred while checking user data. Try reinstall Mobile Assistant Connector');
            });
    }

    function saveUser(user_id, is_new_user, login, password) {
        var actions_codes           = '';
        var actions_codes_container = $('#mobassistantconnector_actions_codes_user_' + user_id);
        var status                  = $('#mobassistantconnector_td_status_user_' + user_id + ' div').attr('val');

        if (actions_codes_container.length > 0) {
            actions_codes = actions_codes_container.val();
        }

        //$.post(baseUrl + 'modules/mobassistantconnector/functions/ajax.php',
        $.post(adminModuleUrl,
            {
                call_function: 'saveUser',
                user_id: user_id,
                is_new_user: is_new_user,
                login: login,
                password: password,
                actions_codes: actions_codes,
                value: status,
                auth_key: key
            },
            function (data) {
                if (data == 'success') {
                    getUserByUsername(login, user_id, is_new_user);
                } else {
                    if (data == "error") {
                        data = "Some error occurred while saving user";
                    }

                    $('#mobassistantconnector_row_loader_user_' + user_id).hide();
                    $('#mobassistantconnector_row_user_' + user_id).show();
                    alert(data);
                }
            }, 'json'
        ).error(function(error) {
                $('#mobassistantconnector_row_loader_user_' + user_id).hide();
                $('#mobassistantconnector_row_user_' + user_id).show();
                alert("Mobile Assistant Connector Error:\r\n" + error.responseText);
            });
    }

    function isNewUser(element) {
        return isNewUserByValue(element.attr('user_id'));
    }

    function isNewUserByValue(value) {
        var is_new_user = '1';

        if (String(value).indexOf('new_') === -1) {
            is_new_user = '0';
        }

        return is_new_user;
    }

    function generateUserRow(data) {
        var status;
        var checkbox = '';
        var actions_codes = '';
        var qr_code = '';
        var save_btn_new_user = '';
        var save_login_password = '';
        var login_prev = '';
        var password_prev = '';
        var devices = '';

        if (data.status == 1) {
            status = '<div class="mobassistantconnector_users_status" user_id="' + data.user_id + '" val="1">' +
                enabled + '</div>';
        } else {
            status = '<div class="mobassistantconnector_users_status" user_id="' + data.user_id + '" val="0">' +
                disabled + '</div>';
        }

        if (isNewUserByValue(data.user_id) == '0') {
            checkbox = '<input type="checkbox" class="mobassistantconnector_users" name="mobassistantconnector_users[]" ' +
                'value="' + data.user_id + '">';

            qr_code = '<div class="mobassistantconnector_td_users_qr_code">' +
                '<img class="mobassistantconnector_users_icon_qr_code" id="mobassistantconnector_img_qr_code_user_' +
                data.user_id + '" user_id="' + data.user_id + '" qr_code_data="' + data.qr_code_data +
                '" title="Get QR-code" class="mobassistantconnector_img_users_qr_code" src="' +
                baseUrl + 'modules/mobassistantconnector/views/img/qrcode.png"><input type="hidden" ' +
                'id="mobassistantconnector_qr_code_link_user_' + data.user_id + '" value="' +
                getQrCodeLink(data.qr_code_hash) + '"></div>';

            save_login_password = '<input class="button btn btn-primary mobassistantconnector_users_save_login_password" ' +
                'id="mobassistantconnector_save_login_password_user_' + data.user_id + '" user_id="' + data.user_id +
                '" type="button" value="Save login/password">';

            login_prev = '<input id="mobassistantconnector_login_prev_user_' + data.user_id +
                '" type="hidden" value="' + data.username + '" size="40">';

            password_prev = '<input id="mobassistantconnector_password_prev_user_' + data.user_id +
                '" type="hidden" value="' + data.password + '" size="40">';

            devices = '<button type="button" class="mobassistantconnector_btn_users_devices btn btn-default" ' +
                'id="mobassistantconnector_btn_devices_user_' + data.user_id + '" user_id="' + data.user_id +
                '">Show devices</button>';
        } else {
            actions_codes = '<input id="mobassistantconnector_actions_codes_user_' + data.user_id +
                '" type="hidden" value="">';

            save_btn_new_user = '<input class="button btn btn-primary mobassistantconnector_users_save" ' +
                'id="mobassistantconnector_save_user_' + data.user_id + '" user_id="' + data.user_id +
                '" type="button" value="Save">';
        }

        return '<td>' + checkbox + '</td><td><table class="mobassistantconnector_users_login_password_table"><tr><td>' +
            '<input class="mobassistantconnector_users_login" id="mobassistantconnector_login_user_' + data.user_id +
            '" user_id="' + data.user_id + '" type="text" value="' + data.username + '" size="40">' + login_prev +
            '</td><td><input class="mobassistantconnector_users_password" id="mobassistantconnector_password_user_' +
            data.user_id + '" user_id="' + data.user_id + '" type="password" value="' + data.password +
            '" size="40">' + password_prev + '</td></tr><tr id="mobassistantconnector_tr_save_login_password_user_' +
            data.user_id + '" class="mobassistantconnector_tr_users_save_login_password"><td colspan="2">' +
            save_login_password + '</td></tr></table><td><div class="mobassistantconnector_users_td_actions" ' +
            'title="Edit allowed actions" user_id="' + data.user_id + '"><div style="width: 100%" ' +
            '<div class="mobassistantconnector_actions_text_loader" id="mobassistantconnector_actions_text_loader_user_' +
            data.user_id + '">' + loader + '</div><button type="button" ' +
            'class="mobassistantconnector_users_td_actions_text btn btn-default pull-right" ' +
            'id="mobassistantconnector_users_td_actions_text_user_' + data.user_id + '" user_id="' + data.user_id +
            '"><table width="100%" class="mobassistantconnector_users_table_actions"><tr>' +
            '<td class="mobassistantconnector_users_actions_text" id="mobassistantconnector_users_actions_text_user_' +
            data.user_id + '"><div class="mobassistantconnector_div_user_actions_text" ' +
            'id="mobassistantconnector_div_actions_text_user_' + data.user_id + '">' + data.allowed_actions +
            '</div></td><td style="width: 20px"><img src="' + baseUrl +
            'modules/mobassistantconnector/views/img/down.png"></td></tr></table></button>' +
            '<div class="mobassistantconnector_users_div_edit_actions">' + actions_codes +
            '</div></div></div></td><td>' + qr_code + '</td><td id="mobassistantconnector_td_status_user_' +
            data.user_id + '" class="mobassistantconnector_center mobassistantconnector_td_users_status">' +
            status + '</td><td class="mobassistantconnector_center">' + devices +
            '</td><td><div class="mobassistantconnector_users_delete" user_id="' + data.user_id +
            '" title="Delete user"><img src="' + baseUrl + 'modules/mobassistantconnector/views/img/trash.png"></div>' +
            '</td><td class="mobassistantconnector_users_save_td">' + save_btn_new_user + '</td>';
    }

    function getUserQrCodeDialogBody() {
        return '<div class="mobassistantconnector_users_qr_code_container" id="mobassistantconnector_qr_code_user">' +
            '<div class="mobassistantconnector_users_qr_code_link_loader"><div>' +
            '<div id="mobassistantconnector_qr_code_scan_user">' +
            '</div></div><div id="mobassistantconnector_qr_code_link_user"></div>' +
            '<div id="mobassistantconnector_qr_code_loader_user">' +
            '<div class="mobassistantconnector_users_qr_code_loader_inner_div"><img src="' + baseUrl +
            'modules/mobassistantconnector/views/img/loader.gif"></div></div></div><div></div>';
    }

    function getUserPermissionsDialogBody() {
        return '<div style="position: relative"><div id="mobassistantconnector_users_actions_container">' +
            '<div id="mobassistantconnector_loader_actions"><div class="mobassistantconnector_loader"><img src="' +
            baseUrl + 'modules/mobassistantconnector/views/img/loader.gif"></div></div>' +
            '<div id="mobassistantconnector_user_actions_container"></div></div>';
    }

    function generateUserRowPushes(pushes, user_id) {
        var row = '<table class="table mobassistantconnector_users_table_pushes" ' +
            'id="mobassistantconnector_table_pushes_user_' + user_id + '"><tr><th>Device Name</th>' +
            '<th>Account Email</th><th>Last Activity</th><th>Account Status</th><th>App Connection ID</th><th>Shop</th>' +
            '<th>New Order</th><th>New Customer</th><th>Order Statuses</th><th>Currency</th><th></th></tr>';
        var is_data_empty = true;

        $.each(pushes, function(index, values) {
            var count_pushes = values.pushes.length;
            is_data_empty = false;

            for (var i = 0; i < count_pushes; i++) {
                var status = '';
                var delimiter = '';
                var center_account_email = '';
                var center_device_name = '';
                var center_statuses = '';
                var enabled = '<img src="' + baseUrl + 'modules/mobassistantconnector/views/img/enabled.gif" ' +
                    'title="Disable ' + values.account_email + '">';
                var disabled = '<img src="' + baseUrl + 'modules/mobassistantconnector/views/img/disabled.gif" ' +
                    'title="Enable ' + values.account_email + '">';
                var new_order = 'no.png';
                var new_customer = 'no.png';

                if (i == (count_pushes - 1)) {
                    delimiter = ' class="mobassistantconnector_device_delimiter" ';
                }

                row += '<tr id="mobassistantconnector_row_push_' + values.pushes[i].id + '"' + delimiter + '>';

                if (i == 0) {
                    if (values.account_email == '-') {
                        center_account_email = ' mobassistantconnector_center';
                    }

                    if (values.device_name == '-') {
                        center_device_name = ' mobassistantconnector_center';
                    }

                    if (values.account_id > 0) {
                        if (values.status == 1) {
                            status = '<div class="mobassistantconnector_status" account_id="' + values.account_id +
                                '" val="1" account_email="' + values.account_email + '">' + enabled + '</div>';
                        } else {
                            status = '<div class="mobassistantconnector_status" account_id="' + values.account_id +
                                '" val="0" account_email="' + values.account_email + '">' + disabled + '</div>';
                        }
                    }

                    row += '<td class="mobassistantconnector_device_delimiter' + center_device_name + '" rowspan="' +
                        count_pushes + '">' + values.device_name + '</td>' +
                        '<td class="mobassistantconnector_device_delimiter ' + center_account_email +
                        '" rowspan="' + count_pushes + '">' + values.account_email +
                        '</td><td class="mobassistantconnector_device_delimiter mobassistantconnector_center" ' +
                        'rowspan="' + count_pushes + '">' + values.last_activity + '</td>' +
                        '<td class="mobassistantconnector_device_delimiter mobassistantconnector_center ' +
                        'mobassistantconnector_status" rowspan="' + count_pushes + '">' + status + '</td>';
                }

                if (values.pushes[i].order_statuses == "-") {
                    center_statuses = 'class="mobassistantconnector_center"';
                }

                if (values.pushes[i].new_order == "1") {
                    new_order = 'yes.png';
                }

                if (values.pushes[i].new_customer == "1") {
                    new_customer = 'yes.png';
                }

                row += '<td class="mobassistantconnector_center">' + values.pushes[i].app_connection_id + '</td><td>' +
                    values.pushes[i].shop + '</td><td class="mobassistantconnector_center"><img src="' + baseUrl +
                    'modules/mobassistantconnector/views/img/' + new_order +
                    '"></td><td class="mobassistantconnector_center"><img src="' + baseUrl +
                    'modules/mobassistantconnector/views/img/' +
                    new_customer + '"></td><td ' + center_statuses + '>' + values.pushes[i].order_statuses +
                    '</td><td class="mobassistantconnector_center">' + values.pushes[i].currency_iso + '</td><td>' +
                    '<div class="mobassistantconnector_center mobassistantconnector_pushes_delete" push_id="' +
                    values.pushes[i].id + '" user_id="' + user_id + '"><img src="' + baseUrl +
                    'modules/mobassistantconnector/views/img/trash.png" title="Delete push notification data"></div></td></tr>';
            }
        });

        if (is_data_empty) {
            row += '<tr><td colspan="11" style="text-align: center">No data</td></tr>';
        }

        row += '</table><div class="mobassistantconnector_pushes_loader" id="mobassistantconnector_pushes_loader_user_' +
            user_id + '">' + loader + '</div>';

        return row;
    }

    function generateUserRowLoader(user_id) {
        return '<tr id="mobassistantconnector_row_loader_user_' + user_id +
            '" style="display: none"><td id="mobassistantconnector_td_loader_user_' + user_id +
            '" colspan="8"><div id="mobassistantconnector_tr_loader_user_' + user_id +
            '" class="mobassistantconnector_tr_loader_user" style="display: block;"><img src="' + baseUrl +
            'modules/mobassistantconnector/views/img/loader.gif"></div></td></tr>';
    }

    function getUserByUsername(login, user_id, is_new_user) {
        var user_row = $('#mobassistantconnector_row_user_' + user_id);
        var user_row_loader = $('#mobassistantconnector_row_loader_user_' + user_id);

        //$.post(baseUrl + 'modules/mobassistantconnector/functions/ajax.php',
        $.post(adminModuleUrl,
            {
                call_function: 'getUserByUsername',
                login: login,
                auth_key: key
            },
            function (data) {
                if (data.hasOwnProperty('user_id')) {
                    if (is_new_user == '1') {
                        user_row.remove();
                        user_row_loader.remove();
                        $('#mobassistantconnector_users_table > tbody:last-child').append(
                            '<tr id="mobassistantconnector_row_user_' + data.user_id + '">' + generateUserRow(data) +
                            '</tr><tr class="mobassistantconnector_users_row_pushes" id="mobassistantconnector_row_pushes_user_' +
                            data.user_id + '"><td colspan="8">' + generateUserRowPushes(data.pushes, data.user_id) +
                            '</td></tr>' + generateUserRowLoader(data.user_id)
                        );
                        preparePermissionsToView(is_new_user, data.user_id);
                    } else {
                        user_row.html(generateUserRow(data));
                        user_row_loader.hide();
                        user_row.show();
                    }

                    $('.mobassistantconnector_users_no_data').remove();
                } else {
//                    data = "Could not get user by username";
                    user_row_loader.hide();
                    user_row.show();
                    alert("Could not get user by username");
                }
            }, 'json'
        ).error(function() {
                user_row_loader.hide();
                user_row.show();
                alert('Some error occurred while getting user by username. Try reinstall Mobile Assistant Connector');
            });
    }

    function regenerateQrCode(user_id) {
        var qr_code_loader = $('#mobassistantconnector_qr_code_loader_user');
        var qr_code_link = $('#mobassistantconnector_qr_code_link_user');

        qr_code_link.hide();
        qr_code_loader.show();

        //$.post(baseUrl + 'modules/mobassistantconnector/functions/ajax.php',
        $.post(adminModuleUrl,
            {
                call_function: 'regenerateQrCodeHash',
                user_id: user_id,
                auth_key: key
            },
            function (data) {
                if (data.hasOwnProperty('hash')) {
                    var qr_code_lnk = getQrCodeLink(data.hash);

                    qr_code_link.html('<a href="' + qr_code_lnk + '" target="_blank">' + qr_code_lnk + '</a>');
                    $("#mobassistantconnector_qr_code_link_user_" + user_id).val(qr_code_lnk);
                    qr_code_loader.hide();
                    qr_code_link.show();
                } else {
                    qr_code_loader.hide();
                    qr_code_link.show();
//                    alert(data);
                    alert("Could not regenerate QR-code");
                }
            }, 'json'
        ).error(function(error) {
                qr_code_loader.hide();
                qr_code_link.show();
                alert("Mobile Assistant Connector Error:\r\n" + error.responseText);
            });
    }

    function getQrCodeLink(hash) {
        return baseUrl +
            'index.php?fc=module&module=mobassistantconnector&controller=mobassistantconnector&call_function=get_qr_code&hash=' +
            hash;
    }

    function processSaveButtonVisibility(user_id) {
        if ($('#mobassistantconnector_login_user_' + user_id).val() != $('#mobassistantconnector_login_prev_user_' + user_id).val()
            || $('#mobassistantconnector_password_user_' + user_id).val() != $('#mobassistantconnector_password_prev_user_' + user_id).val()) {
            $('#mobassistantconnector_tr_save_login_password_user_' + user_id).show();
        } else {
            $('#mobassistantconnector_tr_save_login_password_user_' + user_id).hide();
        }
    }

    function processSelectedAction(element) {
        var user_id                                          = element.attr('user_id');
        var value_selected                                   = element.val();
        var action_products_list                             = 'products_list';
        var action_product_details                           = 'product_details';
        var action_customers_list                            = 'customers_list';
        var action_customer_details                          = 'customer_details';
        var action_abandoned_carts_list                      = 'abandoned_carts_list';
        var action_abandoned_cart_details                    = 'abandoned_cart_details';
        var action_orders_list                               = 'orders_list';
        var action_order_details                             = 'order_details';
        var action_order_details_pdf                         = 'order_details_pdf';
        var action_order_details_products_list_pickup        = 'order_details_products_list_pickup';
        var action_update_order_status                       = 'update_order_status';
        var action_update_order_tracking_number              = 'update_order_tracking_number';
        var action_push_notification_settings                = 'push_notification_settings';
        var action_push_notification_settings_new_order      = 'push_notification_settings_new_order';
        var action_push_notification_settings_new_customer   = 'push_notification_settings_new_customer';
        var action_push_notification_settings_order_statuses = 'push_notification_settings_order_statuses';

        if (element.is(':checked')) {
            if (value_selected == action_product_details) {
                $('#mobassistantconnector_user_action_' + action_products_list).prop('checked', true);
            } else if (value_selected == action_customer_details) {
                $('#mobassistantconnector_user_action_' + action_customers_list).prop('checked', true);
            } else if (value_selected == action_abandoned_cart_details) {
                $('#mobassistantconnector_user_action_' + action_abandoned_carts_list).prop('checked', true);
            } else if (value_selected == action_order_details) {
                $('#mobassistantconnector_user_action_' + action_orders_list).prop('checked', true);
            } else if (
                value_selected == action_order_details_pdf
                || value_selected == action_order_details_products_list_pickup
                || value_selected == action_update_order_status
                || value_selected == action_update_order_tracking_number
            ) {
                $('#mobassistantconnector_user_action_' + action_orders_list).prop('checked', true);
                $('#mobassistantconnector_user_action_' + action_order_details).prop('checked', true);
            } else if (
                    value_selected == action_push_notification_settings_new_order
                    || value_selected == action_push_notification_settings_new_customer
                    || value_selected == action_push_notification_settings_order_statuses
            ) {
                $('#mobassistantconnector_user_action_' + action_push_notification_settings).prop('checked', true);
            }
        } else {
            if (value_selected == action_products_list) {
                $('#mobassistantconnector_user_action_' + action_product_details).prop('checked', false);
            } else if (value_selected == action_customers_list) {
                $('#mobassistantconnector_user_action_' + action_customer_details).prop('checked', false);
            } else if (value_selected == action_abandoned_carts_list) {
                $('#mobassistantconnector_user_action_' + action_abandoned_cart_details).prop('checked', false);
            } else if (value_selected == action_orders_list) {
                $('#mobassistantconnector_user_action_' + action_order_details).prop('checked', false);
                $('#mobassistantconnector_user_action_' + action_order_details_pdf).prop('checked', false);
                $('#mobassistantconnector_user_action_' + action_order_details_products_list_pickup).prop('checked', false);
                $('#mobassistantconnector_user_action_' + action_update_order_status).prop('checked', false);
                $('#mobassistantconnector_user_action_' + action_update_order_tracking_number).prop('checked', false);
            } else if (value_selected == action_order_details) {
                $('#mobassistantconnector_user_action_' + action_order_details_pdf).prop('checked', false);
                $('#mobassistantconnector_user_action_' + action_order_details_products_list_pickup).prop('checked', false);
                $('#mobassistantconnector_user_action_' + action_update_order_status).prop('checked', false);
                $('#mobassistantconnector_user_action_' + action_update_order_tracking_number).prop('checked', false);
            } else if (value_selected == action_push_notification_settings) {
                $('#mobassistantconnector_user_action_' +
                    action_push_notification_settings_new_order).prop('checked', false);
                $('#mobassistantconnector_user_action_' +
                    action_push_notification_settings_new_customer).prop('checked', false);
                $('#mobassistantconnector_user_action_' +
                    action_push_notification_settings_order_statuses).prop('checked', false);
            }
        }
    }

    function showUserPermissionsDialog(user_id) {
        var dialog_obj = $('#mobassistantconnector_users_permissions_dialog');

        $(function() {
            dialog_obj.dialog({
                resizable: false,
                modal: true,
                autoOpen: false,
                buttons: {
                    "Save": function() {
                        if (isNewUserByValue(user_id) == '0') {
                            saveUserActions(user_id);
                        } else {
                            showSelectedActionsForNewUser(user_id);
                        }
                    },
                    Cancel: function() {
                        $(this).dialog('close');
                    }
                }
            });

            dialog_obj.dialog('open');
        });
    }

    function showUserQrCodeDialog(user_id) {
        var dialog_obj = $('#mobassistantconnector_users_qr_code_dialog');

        $(function() {
            dialog_obj.dialog({
                resizable: false,
                modal: true,
                autoOpen: false,
                width: 320,
                buttons: {
                    "Regenerate QR-code link": function() {
                        regenerateQrCode(user_id);
                    },
                    Cancel: function() {
                        $(this).dialog('close');
                    }
                }
            });

            dialog_obj.dialog('open');
        });
    }

    function setDialogPosition(dialog_obj) {
        dialog_obj.dialog("option", "position", dialog_obj.dialog("option", "position"));
    }
});